@@ -57,6 +57,8 @@ PCM_SAMPLE_SIZE = {
 PCM_SAMPLE_RATE = {
     # map with sample rates used in slimproto."""
     # https://wiki.slimdevices.com/index.php/SlimProto_TCP_protocol.html#Command:_.22strm.22
+    # See %pcm_sample_rates in slimserver/Slim/Player/Squeezebox2.pm and
+    # slimserver/Slim/Player/SqueezePlay.pm for definition of sample rates
     11000: b"0",
     22000: b"1",
     44100: b"3",
@@ -65,7 +67,12 @@ PCM_SAMPLE_RATE = {
     12000: b"6",
     16000: b"7",
     24000: b"8",
+    88200: b":",
     96000: b"9",
+    176400: b";",
+    192000: b"<",
+    352800: b"=",
+    384000: b">",
     0: b"?",
 }
 
@@ -101,6 +108,11 @@ FORMAT_BYTE = {
     "aif": b"p",
 }
 
+FALLBACK_MODEL = "Squeezebox"
+FALLLBACK_FIRMWARE = "Unknown"
+FALLBACK_CODECS = ["pcm"]
+FALLBACK_SAMPLE_RATE = 96000
+
 
 class SlimClient:
     """SLIMProto socket client."""
@@ -142,6 +154,9 @@ class SlimClient:
         self._tasks = []
         if self._connected:
             self._connected = False
+            if self._writer.can_write_eof():
+                self._writer.write_eof()
+            self._writer.close()
             self.callback(EventType.PLAYER_DISCONNECTED, self)
 
     @property
@@ -160,24 +175,26 @@ class SlimClient:
         return self._device_type
 
     @property
-    def device_model(self) -> str | None:
+    def device_model(self) -> str:
         """Return device model of the player."""
-        return self._capabilities.get("ModelName", self._capabilities.get("Model"))
+        return self._capabilities.get(
+            "ModelName", self._capabilities.get("Model", FALLBACK_MODEL)
+        )
 
     @property
-    def max_sample_rate(self) -> int | None:
+    def max_sample_rate(self) -> int:
         """Return max sample rate supported by the player."""
-        return self._capabilities.get("MaxSampleRate")
+        return self._capabilities.get("MaxSampleRate", FALLBACK_SAMPLE_RATE)
 
     @property
     def supported_codecs(self) -> List[str]:
         """Return supported codecs by the player."""
-        return self._capabilities.get("SupportedCodecs", ["pcm"])
+        return self._capabilities.get("SupportedCodecs", FALLBACK_CODECS)
 
     @property
-    def firmware(self) -> str | None:
+    def firmware(self) -> str:
         """Return firmware version string for the player."""
-        return self._capabilities.get("Firmware")
+        return self._capabilities.get("Firmware", FALLLBACK_FIRMWARE)
 
     @property
     def device_address(self) -> str:
@@ -229,14 +246,6 @@ class SlimClient:
         """Return uri of currently loaded track."""
         return self._current_url
 
-    async def _initialize_player(self):
-        """Set some startup settings for the player."""
-        # send version
-        await self._send_frame(b"vers", b"7.8")
-        await self._send_frame(b"setd", struct.pack("B", 0))
-        await self._send_frame(b"setd", struct.pack("B", 4))
-        await self.stop()
-
     async def stop(self):
         """Send stop command to player."""
         self._current_url = ""
@@ -314,7 +323,14 @@ class SlimClient:
             port = 80
 
         if scheme == "https" and not self._capabilities.get("CanHTTPS"):
-            raise UnsupportedContentType("Player does not support HTTPS")
+            # most stream urls are available on HTTP too, try to use that instead
+            self.logger.warning(
+                "HTTPS stream requested but player does not support HTTPS, "
+                "trying HTTP instead but playback may fail."
+            )
+            return await self.play_url(
+                url.replace("https", "http"), crossfade, mime_type, send_flush
+            )
 
         if mime_type is None:
             # try to get the audio format from file extension
@@ -445,7 +461,7 @@ class SlimClient:
         )
         await self._send_frame(b"strm", data + server_ip + httpreq)
 
-    def _process_helo(self, data: bytes):
+    async def _process_helo(self, data: bytes):
         """Process incoming HELO event from player (player connected)."""
         self.logger.debug("HELO received: %s", data)
         # player connected, sends helo info message
@@ -456,7 +472,16 @@ class SlimClient:
         self._device_type = DEVICE_TYPE.get(dev_id, "unknown device")
         self._capabilities = parse_capabilities(data)
         self.logger.debug("Player connected: %s", self.player_id)
-        asyncio.create_task(self._initialize_player())
+        # Set some startup settings for the player
+        await self._send_frame(b"vers", b"7.8")
+        await self._send_frame(b"setd", struct.pack("B", 0))
+        await self._send_frame(b"setd", struct.pack("B", 4))
+        await self.stop()
+        # restore last power and volume levels
+        # NOTE: this can be improved by storing the previous volume/power levels
+        # so they can be restored when the player (re)connects.
+        await self.power(self._powered)
+        await self.volume_set(self.volume_level)
         self._connected = True
         self.callback(EventType.PLAYER_CONNECTED, self)
 
@@ -483,10 +508,9 @@ class SlimClient:
         self.callback(EventType.PLAYER_UPDATED, self)
 
     def _process_stat_audg(self, data):
-        """Process incoming stat AUDg message (volume level)."""
-        # TODO: process volume level
-        self.logger.debug("AUDg received - Volume level: %s", data)
-        self.callback(EventType.PLAYER_UPDATED, self)
+        """Process incoming stat AUDg message."""
+        # srtm-s command received.
+        # Some players may send this as aknowledge of volume change (audg command).
 
     def _process_stat_stmc(self, data):
         """Process incoming stat STMc message (connected)."""
@@ -495,13 +519,11 @@ class SlimClient:
     def _process_stat_stmd(self, data):
         """Process incoming stat STMd message (decoder ready)."""
         # pylint: disable=unused-argument
-        self.logger.debug("STMd received - Decoder Ready for next track.")
         self.callback(EventType.PLAYER_DECODER_READY, self)
 
     def _process_stat_stmf(self, data):
         """Process incoming stat STMf message (connection closed)."""
         # pylint: disable=unused-argument
-        self.logger.debug("STMf received - connection closed.")
         self._state = PlayerState.IDLE
         self.callback(EventType.PLAYER_UPDATED, self)
 
@@ -567,7 +589,7 @@ class SlimClient:
             self._last_report = elapsed_seconds
             # send report with elapsed time only every second while playing
             # note that the (very) accurate elapsed/current is always available in the property
-            self.callback(EventType.PLAYER_UPDATED, self)
+            self.callback(EventType.PLAYER_TIME_UPDATED, self)
 
     def _process_stat_stmu(self, data):
         """Process incoming stat STMu message: Buffer underrun: Normal end of playback."""
