@@ -10,6 +10,7 @@ from .cli import SlimProtoCLI
 from .client import SlimClient
 from .const import EventType, SlimEvent
 from .discovery import start_discovery
+from .util import select_free_port
 
 EventCallBackType = Callable[[SlimEvent], None]
 EventSubscriptionType = Tuple[EventCallBackType, Tuple[EventType], Tuple[str]]
@@ -21,18 +22,23 @@ class SlimServer:
     def __init__(
         self,
         port: int = 3483,
-        cli_port: Optional[int] = 9090,
-        cli_port_json: Optional[int] = 3484,
+        cli_port: Optional[int] = 0,
+        cli_port_json: Optional[int] = 0,
     ) -> None:
         """
         Initialize SlimServer instance.
 
         control_port: The TCP port for the slimproto communication, default is 3483.
         cli_port: Optionally start a simple Telnet CLI server on this port for compatability
-        with players relying on the server providing this feature. None to disable.
-        cli_port_json: Same as cli port but it's newer JSOn RPC equivalent.
+        with players relying on the server providing this feature. None to disable, 0 for autoselect.
+        cli_port_json: Same as cli port but it's newer JSON RPC equivalent.
         """
         self.logger = logging.getLogger(__name__)
+        # if port is specified as 0, auto select a free port for the cli/json interface
+        if cli_port == 0:
+            cli_port = select_free_port(9090, 9190)
+        if cli_port_json == 0:
+            cli_port_json = select_free_port(cli_port + 1, 9190)
         self.port = port
         self.cli = SlimProtoCLI(self, cli_port, cli_port_json)
         self._subscribers: List[EventSubscriptionType] = []
@@ -55,7 +61,7 @@ class SlimServer:
             # start slimproto server
             await asyncio.start_server(self._create_client, "0.0.0.0", self.port),
             # setup discovery
-            await start_discovery(self.port, self.cli.cli_port_json),
+            await start_discovery(self.port, self.cli.cli_port, self.cli.cli_port_json),
         ]
         self._socket_servers += await self.cli.start()
 
